package cn.kk.spring_simple_operation.service.impl;

import cn.kk.spring_simple_operation.entity.PhotoProgress;
import cn.kk.spring_simple_operation.mapper.PhotoProgressMapper;
import cn.kk.spring_simple_operation.model.dto.SkuSiteDTO;
import cn.kk.spring_simple_operation.model.vo.PhotoSkuInfo;
import cn.kk.spring_simple_operation.service.PhotoProgressService;
import cn.kk.spring_simple_operation.service.VisualDesignLogService;
import cn.kk.spring_simple_operation.utils.CommonUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentSkipListSet;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * <p>
 * 图片协同系统 服务实现类
 * </p>
 *
 * @author guokunkun
 * @since 2024-12-25
 */
@Slf4j
@Service
public class PhotoProgressServiceImpl extends ServiceImpl<PhotoProgressMapper, PhotoProgress> implements PhotoProgressService {

	@Resource
	private VisualDesignLogService visualDesignLogService;

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void setUploadStatusIsDeleted(List<SkuSiteDTO> list) {
		List<PhotoProgress> listPhotoProgress = baseMapper.getListBySku(list.stream().map(SkuSiteDTO::getSku).collect(Collectors.toList()));
		List<Long> ids = listPhotoProgress.stream()
				.filter(t -> t.getSimpleVideoUploadStatus() != 5)
				.map(PhotoProgress::getId)
				.collect(Collectors.toList());

		update(Wrappers.<PhotoProgress>lambdaUpdate().in(PhotoProgress::getId, ids)
				.set(PhotoProgress::getSimpleVideoUploadStatus, 5)
		);
		visualDesignLogService.addBatchLog("photo_progress", "根据文件将sku的视频上传后台状态设置为已删除", ids);
	}


	@Override
	public void getVideoSkuArea() {
		List<PhotoSkuInfo> skuInfoList = baseMapper.getSkuInfo();
		ConcurrentHashMap<String, Integer> areaCountSingleMap = new ConcurrentHashMap<>();
		areaCountSingleMap.put("深圳", 0);
		areaCountSingleMap.put("武汉", 0);
		areaCountSingleMap.put("合肥", 0);
		areaCountSingleMap.put("长沙", 0);
		ConcurrentHashMap<String, ConcurrentSkipListSet<String>> areaCountMainMap = new ConcurrentHashMap<>();
		areaCountMainMap.put("深圳", new ConcurrentSkipListSet<>());
		areaCountMainMap.put("武汉", new ConcurrentSkipListSet<>());
		areaCountMainMap.put("合肥", new ConcurrentSkipListSet<>());
		areaCountMainMap.put("长沙", new ConcurrentSkipListSet<>());


		skuInfoList.forEach(t -> {
			String region = CommonUtils.getRegion(t.getDept());
			areaCountSingleMap.put(region, areaCountSingleMap.getOrDefault(region, 0) + 1);

			ConcurrentSkipListSet<String> skuList = areaCountMainMap.get(region);
			if (Objects.isNull(skuList)) {
				throw new RuntimeException("区域：" + region + "不存在");
			}

			switch (t.getProductType()) {
				case 1:
					// 普通产品
					if (StringUtils.isEmpty(t.getRelativeSku())) {
						skuList.add(t.getSku());
					}
					else {
						skuList.add(t.getRelativeSku());
					}
					break;
				case 2:
					// 组合产品
					if (t.getIsCombVariant() == 0) {
						//母体
						skuList.add(t.getSku());
					}
					else if (t.getIsCombVariant() == 1) {
						// 变体，添加他的母体
						skuList.add(t.getCombParentSku());
					}
					break;
				case 3:
					skuList.add(t.getRelativeSku());
					break;
				default:
					log.error("产品类型错误：" + t.getProductType());
					break;
			}

		});

		AtomicInteger totalCount = new AtomicInteger(0);
		log.info("区域统计结果：----母体sku总数");
		areaCountMainMap.forEach((k, v) -> {
			log.info("区域：" + k + "，SKU数量：" + v.size());
			totalCount.addAndGet(v.size());
		});
		log.info("合计： " + totalCount.get());

		totalCount.set(0);

		log.info("区域统计结果：----sku总数");
		areaCountSingleMap.forEach((k, v) -> {
			log.info("区域：" + k + "，SKU数量：" + v);
			totalCount.addAndGet(v);
		});
		log.info("合计： " + totalCount.get());

		areaCountMainMap.clear();
		areaCountSingleMap.clear();
		areaCountSingleMap.put("深圳", 0);
		areaCountSingleMap.put("武汉", 0);
		areaCountSingleMap.put("合肥", 0);
		areaCountSingleMap.put("长沙", 0);
		areaCountMainMap.put("深圳", new ConcurrentSkipListSet<>());
		areaCountMainMap.put("武汉", new ConcurrentSkipListSet<>());
		areaCountMainMap.put("合肥", new ConcurrentSkipListSet<>());
		areaCountMainMap.put("长沙", new ConcurrentSkipListSet<>());



		baseMapper.getSkuInfo2().forEach(t -> {
			String region = CommonUtils.getRegion(t.getDept());
			areaCountSingleMap.put(region, areaCountSingleMap.getOrDefault(region, 0) + 1);

			ConcurrentSkipListSet<String> skuList = areaCountMainMap.get(region);
			if (Objects.isNull(skuList)) {
				throw new RuntimeException("区域：" + region + "不存在");
			}

			switch (t.getProductType()) {
				case 1:
					// 普通产品
					if (StringUtils.isEmpty(t.getRelativeSku())) {
						skuList.add(t.getSku());
					}
					else {
						skuList.add(t.getRelativeSku());
					}
					break;
				case 2:
					// 组合产品
					if (t.getIsCombVariant() == 0) {
						//母体
						skuList.add(t.getSku());
					}
					else if (t.getIsCombVariant() == 1) {
						// 变体，添加他的母体
						skuList.add(t.getCombParentSku());
					}
					break;
				case 3:
					skuList.add(t.getRelativeSku());
					break;
				default:
					log.error("产品类型错误：" + t.getProductType());
					break;
			}

		});

		totalCount.set(0);

		log.info("老任务区域统计结果：----母体sku总数");
		areaCountMainMap.forEach((k, v) -> {
			log.info("区域：" + k + "，SKU数量：" + v.size());
			totalCount.addAndGet(v.size());
		});
		log.info("合计： " + totalCount.get());

		totalCount.set(0);

		log.info("老任务区域统计结果：----sku总数");
		areaCountSingleMap.forEach((k, v) -> {
			log.info("区域：" + k + "，SKU数量：" + v);
			totalCount.addAndGet(v);
		});
		log.info("合计： " + totalCount.get());
	}
}
